package main

import (
	"fmt"
	"log"
	"net/http"
	"valera/internal/calories"
	"valera/internal/commands"
	"valera/internal/config"
	"valera/internal/db"
	"valera/internal/pause"
	"valera/internal/states"
	"valera/internal/states/clear_bot_state"
	"valera/internal/states/go_bot_state"
	"valera/internal/states/help_bot_state"
	"valera/internal/states/ping_bot_state"
	"valera/internal/states/start_bot_state"
	"valera/internal/states/stat_bot_state"

	tgbot "github.com/go-telegram-bot-api/telegram-bot-api"
)

const (
	version = "v1.7.0"
)

func main() {
	cfg := config.NewAppConfig()

	bot, err := tgbot.NewBotAPI(cfg.TgConfig.Token)
	if err != nil {
		panic(err)
	}

	dataBase, err := db.NewDB(cfg)
	if err != nil {
		panic(err)
	}

	u := tgbot.NewUpdate(0)
	u.Timeout = 60

	updates, err := bot.GetUpdatesChan(u)
	if err != nil {
		panic(err)
	}

	botStates := []states.BotState{
		clear_bot_state.NewClearBotState(bot, dataBase),
		go_bot_state.NewGoBotState(bot, dataBase),
		stat_bot_state.NewStatBotState(bot, dataBase),
		start_bot_state.NewStartBotState(bot, dataBase, version),
		help_bot_state.NewHelpBotState(bot, dataBase),
		ping_bot_state.NewPingBotState(bot, dataBase),
	}

	go func() {
		for _, s := range botStates {
			http.HandleFunc(s.GetHandler())
		}
		log.Println("Server is start up! port", cfg.Port)
		log.Fatal(http.ListenAndServe(cfg.Port, nil))
	}()

	for update := range updates {

		if update.Message == nil {
			continue
		}

		text := update.Message.Text
		chatID := update.Message.Chat.ID
		username := update.Message.From.UserName
		userInfoDTO, err := dataBase.GetChatInfo(chatID)
		if err != nil {
			log.Println(err)
			continue
		}

		for _, s := range botStates {
			if err := s.Do(text, userInfoDTO, username); err != nil {
				log.Println(err)
			}
		}

		switch userInfoDTO.GetStatus() {
		case db.UserStateEat:
			count, err := calories.CalcCalories(text)
			if err != nil {
				log.Println(err)
				continue
			}
			if err := dataBase.AddCalories(chatID, db.NewCalories(count, username)); err != nil {
				log.Println(err)
				continue
			}
			if count <= 0 {
				_, _ = bot.Send(tgbot.NewMessage(chatID, "Все фигня, давай по новой"))
				continue
			}
			_, _ = bot.Send(tgbot.NewMessage(chatID, fmt.Sprintf("Калории, фу, %d, записал.", count)))
			if err := dataBase.SetStatusToChat(chatID, db.UserStateNone); err != nil {
				log.Println(err)
			}
			continue
		case db.UserStatePause:
			duration, err := pause.GetDuration(text)
			if err != nil {
				log.Println(err)
				continue
			}
			if err := dataBase.SetPause(chatID, duration); err != nil {
				log.Println(err)
			}
			msg := tgbot.NewMessage(chatID, fmt.Sprintf("Поставил паузу %v, отдыхай", duration))
			msg.ReplyMarkup = tgbot.NewRemoveKeyboard(false)
			_, _ = bot.Send(msg)
			if err := dataBase.SetStatusToChat(chatID, db.UserStateNone); err != nil {
				log.Println(err)
			}
			continue
		}

		command := commands.Command(text)
		switch command {
		case commands.Eat:
			if _, err := bot.Send(tgbot.NewMessage(chatID, "Вижу ты поел, отпишись сколько калорий было")); err == nil {
				if err := dataBase.SetStatusToChat(chatID, db.UserStateEat); err != nil {
					log.Println(err)
				}
			}
		case commands.Pause:
			msg := tgbot.NewMessage(chatID, "Хочешь отдохнуть? Сколько времени тебе нужно?")
			msg.ReplyMarkup = tgbot.NewReplyKeyboard([][]tgbot.KeyboardButton{
				tgbot.NewKeyboardButtonRow(
					tgbot.NewKeyboardButton("1ч"),
					tgbot.NewKeyboardButton("4ч"),
				),
				tgbot.NewKeyboardButtonRow(
					tgbot.NewKeyboardButton("1д"),
					tgbot.NewKeyboardButton("2д"),
					tgbot.NewKeyboardButton("7д"),
				),
			}...)
			if _, err := bot.Send(msg); err == nil {
				if err := dataBase.SetStatusToChat(chatID, db.UserStatePause); err != nil {
					log.Println(err)
				}
			}
		}
	}
}
